import 'package:flutter/material.dart';
import 'package:surveytest/components/questionColumn.dart';
import 'package:surveytest/components/skipAndNext.dart';
import 'package:sizer/sizer.dart';

class MultiChoice extends StatefulWidget {
  final Function func;
  final Map<dynamic, dynamic> answer;
  final Map<dynamic, dynamic> question;
  final Map<dynamic, dynamic> theme;
  final Map<String, String> customParams;
  final int currentQuestionNumber;
  const MultiChoice({
    Key? key,
    required this.func,
    required this.answer,
    required this.question,
    required this.theme,
    required this.customParams,
    required this.currentQuestionNumber,
  }) : super(key: key);

  @override
  State<MultiChoice> createState() => _MultiChoiceState(
        func: this.func,
        answer: this.answer,
        question: question,
        theme: this.theme,
        customParams: this.customParams,
        currentQuestionNumber: currentQuestionNumber,
      );
}

class _MultiChoiceState extends State<MultiChoice> {
  Function func;
  final Map<dynamic, dynamic> answer;
  final Map<dynamic, dynamic> question;
  final Map<dynamic, dynamic> theme;
  final Map<String, String> customParams;
  final int currentQuestionNumber;
  _MultiChoiceState({
    required this.func,
    required this.answer,
    required this.question,
    required this.theme,
    required this.customParams,
    required this.currentQuestionNumber,
  });

  @override
  initState() {
    super.initState();
    // print("1189-1 called in multi choice ${question['choices']}");
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        Column(
            mainAxisAlignment: MainAxisAlignment.center,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              QuestionColumn(
                question: question,
                currentQuestionNumber: this.currentQuestionNumber,
                customParams: this.customParams,
                theme: this.theme,
              ),
              MultipleChoiceRow(
                answer: answer,
                func: func,
                question: question,
                theme: theme,
              ),
              const SizedBox(height: 30),
              SkipAndNextButtons(
                showNext: false,
                showSkip: true,
                onClickSkip: () {
                  this.func(null, question['id']);
                  print("skip is clicked");
                },
                theme: theme,
              )
              // ElevatedButton(onPressed: () {}, child: const Text("Submit"))
            ])
      ],
    );
  }
}

class MultipleChoiceRow extends StatefulWidget {
  Function func;
  final Map<dynamic, dynamic> answer;
  final Map<dynamic, dynamic> question;
  final Map<dynamic, dynamic> theme;
  MultipleChoiceRow({
    Key? key,
    required this.func,
    required this.answer,
    required this.question,
    required this.theme,
  }) : super(key: key);

  @override
  State<MultipleChoiceRow> createState() => _MultipleChoiceRowState(
      func: this.func, answer: this.answer, question: question, theme: theme);
}

class _MultipleChoiceRowState extends State<MultipleChoiceRow> {
  Function func;
  final Map<dynamic, dynamic> answer;
  final Map<dynamic, dynamic> question;
  final Map<dynamic, dynamic> theme;
  var _selectedOption = [];
  var choiceToUpdateIs = [];
  bool _isRadioSelected = false;
  var luminanceValue = 0.5;

  _MultipleChoiceRowState(
      {required this.func,
      required this.answer,
      required this.question,
      required this.theme});

  @override
  initState() {
    super.initState();
    luminanceValue =
        this.theme['decodedOpnionBackgroundColorUnSelected'].computeLuminance();
    print("1189-1 called in multi choice inside row ${answer}");
    if (this.answer[this.question['id']] != null) {
      setState(() {
        _selectedOption = this.answer[this.question['id']];
      });
    } else {
      setState(() {
        _selectedOption = [];
      });
    }
  }

  updateOpnionScale(val) {
    var newSelected = _selectedOption;
    newSelected.add(val);
    setState(() {
      _selectedOption = val;
    });
    this.func(val, question['id']);
    print("val to update is ${val}");
  }

  @override
  Widget build(BuildContext context) {
    print("dd 11 ${_selectedOption} ");
    List<Widget> list = List<Widget>.empty(growable: true);
    var charc = 65;
    for (var i = 0; i < question['choices'].length; i++) {
      list.add(
        GestureDetector(
          onTap: () {
            choiceToUpdateIs = [];
            choiceToUpdateIs.add(question['choices'][i]['id']);
            print("choice tapped ${choiceToUpdateIs} ");
            updateOpnionScale(choiceToUpdateIs);
          },
          child: Container(
            constraints: BoxConstraints(maxHeight: 50, maxWidth: 250),
            decoration: BoxDecoration(
              color: _selectedOption.contains(question['choices'][i]['id'])
                  ? this.theme['decodedOpnionBackgroundColorSelected']
                  : this.theme['decodedOpnionBackgroundColorUnSelected'],
              border: Border.all(
                color: this.theme['decodedOpnionBorderColor'],
                width: 1,
              ),
              borderRadius: BorderRadius.circular(8),
            ),
            child: Center(
              child: Container(
                padding: EdgeInsets.only(right: 10, left: 10),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    Text(question['choices'][i]['txt'],
                        textAlign: TextAlign.center,
                        style: TextStyle(
                          decoration: TextDecoration.none,
                          fontSize: 14,
                          fontWeight: FontWeight.w400,
                          color: _selectedOption
                                  .contains(question['choices'][i]['id'])
                              ? luminanceValue > 0.5
                                  ? Colors.black
                                  : Colors.white
                              : this.theme['answerColor'],
                        )),
                    Container(
                      width: 32,
                      height: 32,
                      child: Center(
                        child: Text(String.fromCharCode(charc + i),
                            style: TextStyle(
                              decoration: TextDecoration.none,
                              fontSize: 12,
                              fontWeight: FontWeight.bold,
                              color: luminanceValue > 0.5
                                  ? Colors.black
                                  : Colors.white,
                            )),
                      ),
                      decoration: BoxDecoration(
                        shape: BoxShape.circle,
                        color: this.theme['answerColor'],
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ),
        ),
      );
    }
    return Container(
      child: Wrap(
        crossAxisAlignment: WrapCrossAlignment.center,
        alignment: WrapAlignment.center,
        spacing: 10,
        runSpacing: 10,
        direction: Axis.vertical,
        children: [...list],
      ),
    );

    // Wrap(
    //   crossAxisAlignment: WrapCrossAlignment.center,
    //   alignment: WrapAlignment.center,
    //   spacing: 10,
    //   runSpacing: 10,
    //   direction: Axis.vertical,
    //   children: [...list],
    // );
  }
}
