import 'package:flutter/material.dart';
import 'package:surveytest/components/questionColumn.dart';

import 'skipAndNext.dart';
import 'package:sizer/sizer.dart';

class ColumnOpnionScale extends StatefulWidget {
  final Function func;
  final Map<dynamic, dynamic> answer;
  final Map<dynamic, dynamic> question;
  final Map<dynamic, dynamic> theme;
  final Map<String, String> customParams;
  final int currentQuestionNumber;
  const ColumnOpnionScale({
    Key? key,
    required this.func,
    required this.answer,
    required this.question,
    required this.theme,
    required this.customParams,
    required this.currentQuestionNumber,
  }) : super(key: key);

  @override
  State<ColumnOpnionScale> createState() => _ColumnOpnionScaleState(
      func: this.func,
      answer: this.answer,
      question: question,
      theme: this.theme,
      customParams: this.customParams,
      currentQuestionNumber: currentQuestionNumber);
}

class _ColumnOpnionScaleState extends State<ColumnOpnionScale> {
  Function func;
  final Map<dynamic, dynamic> answer;
  final Map<dynamic, dynamic> question;
  final Map<dynamic, dynamic> theme;
  final Map<String, String> customParams;
  final int currentQuestionNumber;
  _ColumnOpnionScaleState(
      {required this.func,
      required this.answer,
      required this.question,
      required this.theme,
      required this.customParams,
      required this.currentQuestionNumber});
  @override
  Widget build(BuildContext context) {
    // print("119011 updated inside opnion ${currentQuestionNumber} ");
    return Column(
        mainAxisAlignment: MainAxisAlignment.center,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          QuestionColumn(
            question: question,
            currentQuestionNumber: this.currentQuestionNumber,
            customParams: this.customParams,
            theme: this.theme,
          ),
          // SizedBox(height: 40),
          SizedBox(height: 4.h),
          // SizedBox(height: 40),
          OpnionScaleQuestion(
            func: this.func,
            answer: this.answer,
            question: this.question,
            theme: this.theme,
          ),
          SizedBox(height: 7.h),
          // SizedBox(height: 40),
          SkipAndNextButtons(
            showNext: false,
            showSkip: true,
            onClickSkip: () {
              this.func(null, question['id']);
              print("skip is clicked");
            },
            theme: this.theme,
          ),
        ]);
  }
}

class OpnionScaleQuestion extends StatefulWidget {
  final Function func;
  final Map<dynamic, dynamic> answer;
  final Map<dynamic, dynamic> question;
  final Map<dynamic, dynamic> theme;

  const OpnionScaleQuestion({
    Key? key,
    required this.func,
    required this.answer,
    required this.question,
    required this.theme,
  }) : super(key: key);

  @override
  State<OpnionScaleQuestion> createState() => _OpnionScaleQuestionState(
      func: this.func,
      answer: this.answer,
      question: this.question,
      theme: this.theme);
}

class _OpnionScaleQuestionState extends State<OpnionScaleQuestion> {
  Function func;
  final Map<dynamic, dynamic> answer;
  final Map<dynamic, dynamic> question;
  final Map<dynamic, dynamic> theme;
  var luminanceValue = 0.5;

  int _selectedOption = -1;
  _OpnionScaleQuestionState(
      {required this.func,
      required this.answer,
      required this.question,
      required this.theme});

  updateOpnionScale(val) {
    setState(() {
      _selectedOption = val;
    });
    this.func(val, question['id']);
    print("val to update is ${val}");
  }

  @override
  initState() {
    super.initState();
    luminanceValue = this.theme['decodedOpnionBackgroundColorUnSelected'].computeLuminance();
    print(
        "luminance check ${this.theme['decodedOpnionBackgroundColorUnSelected'].computeLuminance()}");
    print("hey color will be ser xxt ${this.theme}");
    // print("called in rating ${this.answer}");
    if (this.answer[this.question['id']] != null) {
      setState(() {
        _selectedOption = this.answer[this.question['id']];
      });
    } else {
      setState(() {
        _selectedOption = -1;
      });
    }
  }

  generateOpmionBlock(val, isSelectedOption) {
    return Container(
        width: 48,
        height: 60,
        // color: val == 0
        //     ? Color.fromARGB(255, 255, 0, 0)
        //     : Color.fromARGB(255, 98, 255, 0),
        margin:
            val == 0 ? EdgeInsets.only(left: 0.0) : EdgeInsets.only(left: 0.0),
        child: Stack(
          clipBehavior: Clip.none,
          children: [
            if (val == 0) ...[
              Positioned(
                top: -8,
                child: Container(
                    child: Text(
                  "Least Likely",
                  style: TextStyle(
                    decoration: TextDecoration.none,
                    fontSize: 12,
                    fontWeight: FontWeight.w400,
                    // color: Color.fromRGBO(67, 67, 67, 1),
                    color: this.theme['decodedOpnionLabelColor'],
                  ),
                )),
              ),
            ] else if (val == 5) ...[
              Positioned(
                  top: -8,
                  child: Container(
                    child: Text(
                      "Neutral",
                      style: TextStyle(
                        decoration: TextDecoration.none,
                        fontSize: 12,
                        fontWeight: FontWeight.w400,
                        // color: Color.fromRGBO(67, 67, 67, 1),
                        color: this.theme['decodedOpnionLabelColor'],
                      ),
                    ),
                  )),
            ] else if (val == 10) ...[
              Positioned(
                  bottom: -8,
                  child: Container(
                    child: Text(
                      "Most Likely",
                      style: TextStyle(
                        decoration: TextDecoration.none,
                        fontSize: 12,
                        fontWeight: FontWeight.w400,
                        color: this.theme['decodedOpnionLabelColor'],
                      ),
                    ),
                  )),
            ],
            Center(
              child: Container(
                // color: Color.fromARGB(255, 255, 247, 0),
                width: 40,
                height: 40,
                // constraints: BoxConstraints(
                //    maxHeight: 40,maxWidth: 40),
                decoration: BoxDecoration(
                  color: isSelectedOption == val
                      ? this.theme['decodedOpnionBackgroundColorSelected']
                      : this.theme['decodedOpnionBackgroundColorUnSelected'],
                  border: Border.all(
                    // color: Color.fromRGBO(63, 63, 63, 0.5),
                    color: this.theme['decodedOpnionBorderColor'],
                    width: 1,
                  ),
                  borderRadius: BorderRadius.circular(8),
                ),
                child: Center(
                  child: Text(val.toString(),
                      textAlign: TextAlign.center,
                      style: TextStyle(
                        decoration: TextDecoration.none,
                        fontSize: 13,
                        fontWeight: FontWeight.bold,
                        color: isSelectedOption == val
                            ? luminanceValue > 0.5 
                                ? Colors.black
                                : Colors.white
                            : this.theme['answerColor'],
                      )),
                ),
              ),
            )
          ],
        ));
  }

  @override
  Widget build(BuildContext context) {
    List<Widget> list = List<Widget>.empty(growable: true);
    for (var i = 0; i < 11; i++) {
      list.add(
        GestureDetector(
          onTap: () {
            print("on tap called for op ${i}");
            updateOpnionScale(i);
          },
          child: Container(
            child: generateOpmionBlock(i, _selectedOption),
          ),
        ),
      );
    }
    return Container(
      width: double.infinity,
      child: Wrap(
        crossAxisAlignment: WrapCrossAlignment.center,
        alignment: WrapAlignment.center,
        // spacing: 10,
        // runSpacing: 20,
        direction: Axis.horizontal,
        children: [...list],
      ),
    );
  }
}
