import 'package:flutter/material.dart';
import 'package:flutter_rating_stars/flutter_rating_stars.dart';
import 'package:surveytest/components/questionColumn.dart';
import 'package:surveytest/helpers/question.dart';

import 'skipAndNext.dart';

class ColumnRating extends StatefulWidget {
  final Function func;
  final Map<dynamic, dynamic> answer;
  final Map<dynamic, dynamic> question;
  final Map<dynamic, dynamic> theme;
  final Map<String, String> customParams;
  final int currentQuestionNumber;
  final bool isLastQuestion;
  const ColumnRating({
    Key? key,
    required this.func,
    required this.answer,
    required this.question,
    required this.theme,
    required this.customParams,
    required this.currentQuestionNumber,
    required this.isLastQuestion,
  }) : super(key: key);

  @override
  State<ColumnRating> createState() => _ColumnRatingState(
        func: this.func,
        answer: this.answer,
        question: question,
        theme: this.theme,
        customParams: this.customParams,
        currentQuestionNumber: currentQuestionNumber,
        isLastQuestion:isLastQuestion,
      );
}

class _ColumnRatingState extends State<ColumnRating> {
  Function func;
  final Map<dynamic, dynamic> answer;
  final Map<dynamic, dynamic> question;
  final Map<dynamic, dynamic> theme;
  final Map<String, String> customParams;
  final int currentQuestionNumber;
  final bool isLastQuestion;
  _ColumnRatingState({
    required this.func,
    required this.answer,
    required this.question,
    required this.theme,
    required this.customParams,
    required this.currentQuestionNumber,
    required this.isLastQuestion,
  });

  @override
  initState() {
    super.initState();
    print("rating test ${this.isLastQuestion}");
    print("draft view testing is done 0093 ${this.question['rtxt']}");
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        //  Text(parsedHeading(this.question,this.customParams),
        //     textAlign: TextAlign.center,
        //     style: TextStyle(decoration: TextDecoration.none, fontSize: 25, color: theme['questionColor'])),
        // SizedBox(height: 10),
        // Text(this.question['rdesc']['blocks'][0]['text'],
        //     textAlign: TextAlign.center,
        //     style: TextStyle(decoration: TextDecoration.none, fontSize: 18,color: theme['questionColor'])),
        // SizedBox(height: 30),
        QuestionColumn(
          question: question,
          currentQuestionNumber: this.currentQuestionNumber,
          customParams: this.customParams,
          theme: this.theme,
        ),
        RatingQuestion(
          func: this.func,
          answer: this.answer,
          question: this.question,
          theme: this.theme,
        ),
        SizedBox(height: 40),
        SkipAndNextButtons(
          showNext: false,
          showSkip: true,
          onClickSkip: () {
            this.func(null, question['id']);
            print("skip is clicked");
          },
          theme: theme,
        ),
      ],
    );
  }
}

class RatingQuestion extends StatefulWidget {
  final Function func;
  final Map<dynamic, dynamic> answer;
  final Map<dynamic, dynamic> question;
  final Map<dynamic, dynamic> theme;
  const RatingQuestion(
      {Key? key,
      required this.func,
      required this.answer,
      required this.question,
      required this.theme,
      })
      : super(key: key);

  @override
  State<RatingQuestion> createState() => _RatingQuestionState(
      func: this.func, answer: this.answer, question: this.question, theme: this.theme);
}

class _RatingQuestionState extends State<RatingQuestion> {
  Function func;
  final Map<dynamic, dynamic> answer;
  final Map<dynamic, dynamic> question;
  final Map<dynamic, dynamic> theme;
  double _rating = -1.0;
  double widget1Opacity = 0.0;

  _buildRatingStart(int index) {
    print("1q rating build called ${_rating} ${index}");
    if (_rating >= index.toDouble()) {
      return Column(
        children: [
          Icon(
            Icons.star_rounded,
            size: 60,
            color: this.theme['answerColor'],
          ),
          AnimatedOpacity(
            opacity: widget1Opacity,
            duration: const Duration(milliseconds: 200),
            child: Text((index + 1).toString(),style: TextStyle(color:this.theme['decodedOpnionLabelColor'] ),),
          )
        ],
      );
    }
    return Column(
      children: [
        Icon(Icons.star_border_rounded,
            size: 60, color: this.theme['decodedOpnionBorderColor']),
        AnimatedOpacity(
          opacity: widget1Opacity,
          duration: const Duration(milliseconds: 200),
          child: Text((index + 1).toString(),style: TextStyle(color:this.theme['decodedOpnionLabelColor'] ),),
        )
      ],
    );
  }

  @override
  initState() {
    super.initState();
    print("called in rating ${this.answer}");
    print("question is ${this.question['rdesc']['blocks'][0]['text']}");
    if (this.answer[this.question['id']] != null) {
      setState(() {
        _rating = this.answer[this.question['id']];
      });
    }
    Future.delayed(Duration(milliseconds: 400), () {
      setState(() {
        widget1Opacity = 1.0;
      });
    });
  }

  _RatingQuestionState(
      {required this.func, required this.answer, required this.question, required this.theme});

  @override
  Widget build(BuildContext context) {
    final stars = List<Widget>.generate(5, (index) {
      return GestureDetector(
        onTap: () {
          setState(() {
            _rating = index.toDouble();
          });
          this.func(index.toDouble(), question['id']);
        },
        child: _buildRatingStart(index),
      );
    });

    return Row(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [...stars],
    );
  }
}

class RatingSvg extends StatefulWidget {
  const RatingSvg({Key? key}) : super(key: key);

  @override
  State<RatingSvg> createState() => _RatingSvgState();
}

class _RatingSvgState extends State<RatingSvg> {
  @override
  Widget build(BuildContext context) {
    return Container();
  }
}
