import 'package:flutter/material.dart';
import 'package:flutter_rating_stars/flutter_rating_stars.dart';

class ColumnRating extends StatefulWidget {
  final Function func;
  final Map<dynamic, dynamic> answer;
  final Map<dynamic, dynamic> question;
  const ColumnRating({Key? key, required this.func, required this.answer, required this.question}) : super(key: key);

  @override
  State<ColumnRating> createState() => _ColumnRatingState(func: this.func, answer: this.answer, question: question);
}

class _ColumnRatingState extends State<ColumnRating> {
  Function func;
  final Map<dynamic, dynamic> answer;
  final Map<dynamic, dynamic> question;
  _ColumnRatingState({required this.func, required this.answer, required this.question});
  
  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
         Text(this.question['rtxt']['blocks'][0]['text'],
            textAlign: TextAlign.center,
            style: TextStyle(decoration: TextDecoration.none, fontSize: 25)),
        SizedBox(height: 10),
        Text(this.question['rdesc']['blocks'][0]['text'],
            textAlign: TextAlign.center,
            style: TextStyle(decoration: TextDecoration.none, fontSize: 18)),
        SizedBox(height: 30),
        RatingQuestion(
          func: this.func,
          answer: this.answer,
          question: this.question,
        ),
        SizedBox(height: 30),
      ],
    );
  }
}

class RatingQuestion extends StatefulWidget {
  final Function func;
  final Map<dynamic, dynamic> answer;
  final Map<dynamic, dynamic> question;
  const RatingQuestion({Key? key, required this.func, required this.answer, required this.question})
      : super(key: key);

  @override
  State<RatingQuestion> createState() =>
      _RatingQuestionState(func: this.func, answer: this.answer, question: this.question);
}

class _RatingQuestionState extends State<RatingQuestion> {
  Function func;
  final Map<dynamic, dynamic> answer;
  final Map<dynamic, dynamic> question;
  double _rating = 0;

  @override
  initState() {
    super.initState();
    print("called in rating ${this.answer}");
    print("question is ${this.question['rdesc']['blocks'][0]['text']}");
    if (this.answer[this.question['id']] != null) {
      setState(() {
        _rating = this.answer[this.question['id']];
      });
    } else {
      setState(() {
        _rating = 0.0;
      });
    }
  }

  _RatingQuestionState({required this.func, required this.answer, required this.question});

  @override
  Widget build(BuildContext context) {
    return RatingStars(
      value: _rating,
      onValueChanged: (v) {
        setState(() {
          _rating = v;
        });
        this.func(v, question['id']);
        print("value is ${v}");
      },
      starBuilder: (index, color) => Icon(
        Icons.star,
        size: 50.0,
        color: color,
      ),
      starCount: 5,
      starSize: 40,
      valueLabelRadius: 10,
      maxValue: 5,
      starSpacing: 30,
      maxValueVisibility: true,
      valueLabelVisibility: false,
      animationDuration: const Duration(milliseconds: 200),
      valueLabelPadding: const EdgeInsets.symmetric(vertical: 1, horizontal: 8),
      valueLabelMargin: const EdgeInsets.only(right: 8),
      starOffColor: const Color(0xffe7e8ea),
      starColor: Colors.yellow,
    );
  }
}
