import 'package:flutter/material.dart';

class SkipAndNextButtons extends StatefulWidget {
  final bool showNext;
  final bool showSkip;
  final bool? showSubmit;
  final Function? onClickNext;
  final Function? onClickSkip;
  final Map<dynamic, dynamic> theme;
  const SkipAndNextButtons({
    Key? key,
    required this.showNext,
    required this.showSkip,
    this.onClickNext,
    this.onClickSkip,
    required this.theme,
    this.showSubmit,
  }) : super(key: key);

  @override
  State<SkipAndNextButtons> createState() => _SkipAndNextButtonsState(
        showNext: this.showNext,
        showSkip: this.showSkip,
        onClickNext: this.onClickNext,
        onClickSkip: this.onClickSkip,
        theme: this.theme,
        showSubmit: showSubmit,
      );
}

class _SkipAndNextButtonsState extends State<SkipAndNextButtons> {
  final bool showNext;
  final bool showSkip;
  final Function? onClickNext;
  final Function? onClickSkip;
  final Map<dynamic, dynamic> theme;
  final bool? showSubmit;
  double luminanceValue = 0.5;

  _SkipAndNextButtonsState({
    required this.showNext,
    required this.showSkip,
    this.onClickNext,
    this.onClickSkip,
    required this.theme,
    this.showSubmit,
  });

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    luminanceValue = this.theme['ctaButtonColor'].computeLuminance();
  }

  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.start,
      children: [
        if (showNext) ...[
          ElevatedButton(
            onPressed: () {
              if (onClickNext != null) {
                onClickNext!();
              }
            },
            style: ElevatedButton.styleFrom(
              primary: theme['ctaButtonColor'],
              padding: EdgeInsets.all(10),
              shape: StadiumBorder(),
            ),
            child: Container(
              width: 100,
              child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceAround,
                children: [
                  Text(
                    (this.showSubmit != null && this.showSubmit == true) ? "SUBMIT" : "NEXT",
                    style: TextStyle(
                      decoration: TextDecoration.none,
                      fontSize: 14,
                      fontWeight: FontWeight.bold,
                      color: luminanceValue > 0.5 ? Colors.black : Colors.white,
                    ),
                  ),
                  Icon(
                    (this.showSubmit != null && this.showSubmit == true) ? Icons.check_rounded : Icons.keyboard_arrow_right,
                    color: luminanceValue > 0.5 ? Colors.black : Colors.white,
                  
                  ),
                ],
              ),
            ),
          ),
          SizedBox(width: 20),
        ],
        if (showSkip) ...[
          GestureDetector(
            onTap: () {
              if (onClickSkip != null) {
                onClickSkip!();
              }
            },
            child: Container(
              // width: double.infinity,
              child: Text(
                'SKIP',
                textAlign: TextAlign.left,
                style: TextStyle(
                  decoration: TextDecoration.none,
                  fontSize: 12,
                  fontWeight: FontWeight.bold,
                  color: theme['questionNumberColor'],
                ),
              ),
            ),
          ),
        ],
      ],
    );
  }
}
