import 'package:flutter/material.dart';
import 'package:surveytest/components/questionColumn.dart';

import 'skipAndNext.dart';

class TextRating extends StatefulWidget {
  final Function func;
  final Map<dynamic, dynamic> answer;
  final Map<dynamic, dynamic> question;
  final Map<dynamic, dynamic> theme;
  final Map<String, String> customParams;
  final int currentQuestionNumber;
  final bool isLastQuestion;
  const TextRating({
    Key? key,
    required this.func,
    required this.answer,
    required this.question,
    required this.theme,
    required this.customParams,
    required this.currentQuestionNumber,
    required this.isLastQuestion,
  }) : super(key: key);

  @override
  State<TextRating> createState() => _TextRatingState(
        func: this.func,
        answer: this.answer,
        question: question,
        theme: this.theme,
        customParams: this.customParams,
        currentQuestionNumber: currentQuestionNumber,
        isLastQuestion: isLastQuestion,
      );
}

class _TextRatingState extends State<TextRating> {
  Function func;
  final Map<dynamic, dynamic> answer;
  final Map<dynamic, dynamic> question;
  final Map<dynamic, dynamic> theme;
  final Map<String, String> customParams;
  final int currentQuestionNumber;
  final bool isLastQuestion;

  TextEditingController inputController = new TextEditingController();
  _TextRatingState({
    required this.func,
    required this.answer,
    required this.question,
    required this.theme,
    required this.customParams,
    required this.currentQuestionNumber,
    required this.isLastQuestion,
  });

  @override
  initState() {
    super.initState();
    print("test text ${this.isLastQuestion}");
    if (this.answer[this.question['id']] != null) {
      setState(() {
        inputController.text = this.answer[this.question['id']];
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        SingleChildScrollView(
          child: Column(
              mainAxisAlignment: MainAxisAlignment.center,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                QuestionColumn(
                  question: question,
                  currentQuestionNumber: this.currentQuestionNumber,
                  customParams: this.customParams,
                  theme: this.theme,
                ),
                TextField(
                  style: TextStyle(color: this.theme['answerColor']),
                  cursorColor: this.theme['answerColor'],
                  controller: inputController,
                  decoration: InputDecoration(
                    focusedBorder: UnderlineInputBorder(
                      borderSide: BorderSide(color: this.theme['answerColor']),
                    ),
                    hintText: "Please Enter Your Response",
                    hintStyle:
                        TextStyle(color: this.theme['questionNumberColor']),
                    enabledBorder: UnderlineInputBorder(
                      borderSide: BorderSide(
                        color: this.theme['questionDescriptionColor'],
                      ),
                    ),
                  ),
                ),
                const SizedBox(height: 30),
                SkipAndNextButtons(
                  showNext: true,
                  showSkip: true,
                  onClickNext: () {
                    print(
                        "obtained test input is 1134 ${inputController.text}");
                    this.func(inputController.text, question['id']);
                  },
                  onClickSkip: () {
                    this.func(null, question['id']);
                    print("skip is clicked");
                  },
                  theme: theme,
                ),
              ]),
        )
      ],
    );
  }
}
