library surveytest;

import 'package:eval_ex/expression.dart';
import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;
import 'package:surveytest/components/bottomNavigation.dart';
import 'package:surveytest/components/loadingScreen.dart';
import 'package:surveytest/components/multichoice.dart';
import 'package:surveytest/components/phone.dart';
import 'package:surveytest/models/theme.dart';
import 'dart:convert';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:surveytest/svg.dart';

import 'components/opnion.dart';
import 'components/rating.dart';
import 'components/text.dart';

import 'package:sizer/sizer.dart';

class QuestionWrapper extends StatelessWidget {
  final Map<String, String> customParams;
  final dynamic? firstQuestionAnswer;
  final Function? currentlyCollectedAnswers;
  QuestionWrapper(
      {Key? key,
      required this.customParams,
      this.firstQuestionAnswer,
      this.currentlyCollectedAnswers})
      : super(key: key);

  late Future<Map<dynamic, dynamic>> testeru = fetchAlbum();

  @override
  Widget build(BuildContext context) {
    return Container(
      child: FutureBuilder(
        future: testeru,
        builder: (BuildContext context, AsyncSnapshot snapshot) {
          if (snapshot.hasData) {
            return Sizer(
              builder: (context, orientation, deviceType) {
                return QuestionsPage(
                  Questions: snapshot.data,
                  customParams: customParams,
                  firstQuestionAnswer: firstQuestionAnswer,
                  currentlyCollectedAnswers: currentlyCollectedAnswers,
                );
              },
            );
          }
          return LoadingScreen();
        },
      ),
    );
  }
}

class QuestionsPage extends StatefulWidget {
  final Map<dynamic, dynamic> Questions;
  final Map<String, String> customParams;
  final dynamic? firstQuestionAnswer;
  final Function? currentlyCollectedAnswers;
  const QuestionsPage({
    Key? key,
    required this.Questions,
    required this.customParams,
    this.firstQuestionAnswer,
    this.currentlyCollectedAnswers,
  }) : super(key: key);
  @override
  State<QuestionsPage> createState() => _QuestionsPageState(
      this.Questions, this.customParams, this.firstQuestionAnswer);
}

class _QuestionsPageState extends State<QuestionsPage>
    with TickerProviderStateMixin {
  final Map<dynamic, dynamic> Survey;
  final Map<String, String> customParams;
  final Map<dynamic, dynamic> _questionPos = {};
  final Map<dynamic, dynamic> _surveyToMap = {};
  final dynamic? firstQuestionAnswer;
  _QuestionsPageState(this.Survey, this.customParams, this.firstQuestionAnswer);
  List<Map<dynamic, dynamic>> _allQuestionList = [];
  List _allowedQuestionIds = [];
  Map<dynamic, dynamic> _myMap = {};
  Map<dynamic, dynamic> _themeData = {};
  Map<dynamic,dynamic> _lastQuestion = {};
  var _surveyThemeClass;
  int _currentQuestionNumber = 1;
  Map<dynamic, dynamic> _currentQuestionToRender = {};

  int _pageNumber = 0;
  double _progressMade = 0.0;
  int _answeredCount = 0;
  List<Widget> questionList = new List<Widget>.empty(growable: true);
  final allowedQuestionTypes = [
    'OpinionScale',
    'Rating',
    'TextInput',
    'MultiChoice',
    'PhoneNumber',
  ];

  refresh(value, key) {
    print(" 1134s called in parent ${value} ${key} ");

    if (value == null) {
      if (_myMap[key] != null) {
        setState(() {
          _myMap.remove(key);
        });
      }
      widget.currentlyCollectedAnswers!(_myMap);
    }
    if (value != null) {
      _answeredCount += 1;
      _myMap[key] = value;
      setState(() {
        _myMap[key] = value;
      });
      widget.currentlyCollectedAnswers!(_myMap);
    }

    Future.delayed(const Duration(milliseconds: 500), () {
      _handleNextQuestion();
    });

    print("checking last question 11901 ${_lastQuestion} ${key} ");
  }

  late AnimationController _controller;
  late Animation<double> _animation;
  @override
  initState() {
    super.initState();

    // print("the inital height I got is h11 ${MediaQuery.of(context).size.height * 0.70}");

    _controller =
        AnimationController(duration: Duration(seconds: 3), vsync: this);
    _animation = Tween<double>(begin: 0, end: 1).animate(_controller);

    if (this.Survey['surveyTheme'] != null) {
      _surveyThemeClass =
          SurveyThemeData.fromJson(json: this.Survey['surveyTheme']);
    } else {
      _surveyThemeClass = SurveyThemeData.fromJson();
    }
    _themeData['questionColor'] = _surveyThemeClass.questionColor;
    _themeData['answerColor'] = _surveyThemeClass.answerColor;
    _themeData['backgroundColor'] = _surveyThemeClass.backgroundColor;
    _themeData['questionDescriptionColor'] =
        _surveyThemeClass.questionDescriptionColor;
    _themeData['questionNumberColor'] = _surveyThemeClass.questionNumberColor;
    _themeData['decodedOpnionBackgroundColorSelected'] =
        _surveyThemeClass.decodedOpnionBackgroundColorSelected;
    _themeData['decodedOpnionBackgroundColorUnSelected'] =
        _surveyThemeClass.decodedOpnionBackgroundColorUnSelected;
    _themeData['decodedOpnionBorderColor'] =
        _surveyThemeClass.decodedOpnionBorderColor;
    _themeData['decodedOpnionLabelColor'] =
        _surveyThemeClass.decodedOpnionLabelColor;
    _themeData['ctaButtonColor'] = _surveyThemeClass.ctaButtonColor;

    var _index = 0;
    for (var section in this.Survey['sections']) {
      for (var question in section['questions']) {
        print("1189 multiple cjoice ${question['type']}");
        if (allowedQuestionTypes.contains(question['type'])) {
          _questionPos[question['id']] = _index;
          _surveyToMap[question['id']] = question;
          _allQuestionList.add(question);
          _allowedQuestionIds.add(question['id']);
          _index = _index + 1;
        }
      }
    }

    print('all question list is ${_allowedQuestionIds}');

    print('current question in init -> 1109 ${_allQuestionList[_pageNumber]}');

    setState(() {
      questionList = _convertQuestionListToWidget(_allQuestionList);
      _currentQuestionToRender = _allQuestionList[_pageNumber];
    });

    print("first question answer is ${firstQuestionAnswer}");
    if (firstQuestionAnswer != null) {
      this.refresh(firstQuestionAnswer, _allowedQuestionIds[0]);
    }
    // this.refresh(4.0, _allowedQuestionIds[0]);
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  // _convertSingleQuestionToWidget(questionsToConvert) {
  //   if (questionsToConvert['type'] == 'Rating') {
  //     return (AnimatedOpacity(
  //         opacity: 0.0,
  //         duration: const Duration(milliseconds: 1000),
  //         child: ColumnRating(
  //             func: refresh,
  //             answer: _myMap,
  //             question: questionsToConvert,
  //             theme: _themeData,
  //             customParams: customParams,
  //             currentQuestionNumber: _currentQuestionNumber)));
  //   }
  //   if (questionsToConvert['type'] == 'TextInput') {
  //     return (TextRating(
  //         func: refresh,
  //         answer: _myMap,
  //         question: questionsToConvert,
  //         theme: _themeData,
  //         customParams: customParams,
  //         currentQuestionNumber: _currentQuestionNumber));
  //   }
  //   if (questionsToConvert['type'] == 'MultiChoice') {
  //     return (MultiChoice(
  //       func: refresh,
  //       answer: _myMap,
  //       question: questionsToConvert,
  //       theme: _themeData,
  //       customParams: customParams,
  //       currentQuestionNumber: _currentQuestionNumber,
  //     ));
  //   }
  //   if (questionsToConvert['type'] == 'OpinionScale') {
  //     return (ColumnOpnionScale(
  //         func: refresh,
  //         answer: _myMap,
  //         question: questionsToConvert,
  //         theme: _themeData,
  //         customParams: customParams,
  //         currentQuestionNumber: _currentQuestionNumber));
  //   }
  // }

  _convertQuestionListToWidget(questionsToConvert) {
    List<Widget> _newquestionList = List<Widget>.empty(growable: true);

    for (var question in questionsToConvert) {
      // print("is last question ${question['id']} ${} ");
      print(
          "is current question 1232${_currentQuestionToRender['id'] == question['id']} ${_currentQuestionToRender['id']} ${question['id']}");
      if (question['type'] == 'Rating') {
        _newquestionList.add(
          AnimatedOpacity(
            opacity: _currentQuestionToRender['id'] == null
                ? 1.0
                : _currentQuestionToRender['id'] == question['id']
                    ? 1.0
                    : 0.0,
            duration: const Duration(milliseconds: 200),
            child: Center(
              child: SingleChildScrollView(
                child: ColumnRating(
                  func: refresh,
                  answer: _myMap,
                  question: question,
                  theme: _themeData,
                  customParams: customParams,
                  currentQuestionNumber: _currentQuestionNumber,
                  isLastQuestion:
                      question['id'] == questionsToConvert.last['id'],
                ),
              ),
            ),
          ),
        );
      }
      if (question['type'] == 'TextInput') {
        _newquestionList.add(
          AnimatedOpacity(
            opacity: _currentQuestionToRender['id'] == null
                ? 1.0
                : _currentQuestionToRender['id'] == question['id']
                    ? 1.0
                    : 0.0,
            duration: const Duration(milliseconds: 200),
            child: TextRating(
              func: refresh,
              answer: _myMap,
              question: question,
              theme: _themeData,
              customParams: customParams,
              currentQuestionNumber: _currentQuestionNumber,
              isLastQuestion: question['id'] == questionsToConvert.last['id'],
            ),
          ),
        );
      }
      if (question['type'] == 'MultiChoice') {
        _newquestionList.add(
          AnimatedOpacity(
            opacity: _currentQuestionToRender['id'] == null
                ? 1.0
                : _currentQuestionToRender['id'] == question['id']
                    ? 1.0
                    : 0.0,
            duration: const Duration(milliseconds: 200),
            child: Center(
              child: SingleChildScrollView(
                child: MultiChoice(
                  func: refresh,
                  answer: _myMap,
                  question: question,
                  theme: _themeData,
                  customParams: customParams,
                  currentQuestionNumber: _currentQuestionNumber,
                ),
              ),
            ),
          ),
        );
      }
      if (question['type'] == 'OpinionScale') {
        _newquestionList.add(
          AnimatedOpacity(
            opacity: _currentQuestionToRender['id'] == null
                ? 1.0
                : _currentQuestionToRender['id'] == question['id']
                    ? 1.0
                    : 0.0,
            duration: const Duration(milliseconds: 200),
            child: Center(
              child: SingleChildScrollView(
                child: ColumnOpnionScale(
                    func: refresh,
                    answer: _myMap,
                    question: question,
                    theme: _themeData,
                    customParams: customParams,
                    currentQuestionNumber: _currentQuestionNumber),
              ),
            ),
          ),
        );
      }
      if (question['type'] == 'PhoneNumber') {
        _newquestionList.add(
          AnimatedOpacity(
            opacity: _currentQuestionToRender['id'] == null
                ? 1.0
                : _currentQuestionToRender['id'] == question['id']
                    ? 1.0
                    : 0.0,
            duration: const Duration(milliseconds: 200),
            child: ColumnPhone(
              func: refresh,
              answer: _myMap,
              question: question,
              theme: _themeData,
              customParams: customParams,
              currentQuestionNumber: _currentQuestionNumber,
              isLastQuestion: question['id'] == questionsToConvert.last['id'],
            ),
          ),
        );
      }
    }

    return _newquestionList;
  }

  _greaterThanForRating(logic) {
    var _questionIdToCheck = logic['question_id'];
    if (_myMap[_questionIdToCheck] != null) {
      var answer = _myMap[_questionIdToCheck];
      if (answer > int.parse(logic['value'])) {
        return true;
      } else {
        return false;
      }
    } else {
      return false;
    }
  }

  _lessThanForRating(logic) {
    var _questionIdToCheck = logic['question_id'];
    if (_myMap[_questionIdToCheck] != null) {
      var answer = _myMap[_questionIdToCheck];
      print("logic value called is ${logic} ${answer} ");
      if (logic['value'].length > 0 && answer < int.parse(logic['value'])) {
        return true;
      } else {
        return false;
      }
    } else {
      return false;
    }
  }

  _isSelected(logic) {
    var _questionIdToCheck = logic['question_id'];
    if (_myMap[_questionIdToCheck] != null) {
      var answer = _myMap[_questionIdToCheck];
      print("the choice is 11901 ${answer} ${logic['choice_id']} ");
      if (logic['choice_id'] != null) {
        if (answer.contains(logic['choice_id'])) {
          return true;
        } else {
          return false;
        }
      } else {
        return false;
      }
    } else {
      return false;
    }
  }

  _checkingTheLogics(logic) {
    print("logic comparator is ${logic['comparator']}");
    if (logic['comparator'] == 'greaterThanForRating' ||
        logic['comparator'] == 'greaterThanForScale' ||
        logic['comparator'] == 'greaterThanForNumber') {
      return _greaterThanForRating(logic);
    }
    if (logic['comparator'] == 'lessThanForRating' ||
        logic['comparator'] == 'lessThanForScale') {
      return _lessThanForRating(logic);
    }
    if (logic['comparator'] == 'isSelected') {
      return _isSelected(logic);
    }

    return true;
  }

  _coreDisplayLogic(question, type) {
    var questionLogics;
    if (question == null) {
      return true;
    } else {
      var contionMapping = {'and': '&&', 'or': '||'};
      // return false if the logic is not met else return true if logic is met
      if (type == 'dl') {
        if (question['displayLogic']['logics'].length > 0) {
          questionLogics = question['displayLogic']['logics'];
          var logicEvaluation = [];
          for (var logic in questionLogics) {
            if (_allowedQuestionIds.contains(logic['question_id'])) {
              var join_condition = logic['join_condition'];
              var result = _checkingTheLogics(logic);
              logicEvaluation
                  .add({'join_condition': join_condition, 'result': result});
            }
          }
          var evalString;
          var index = 0;
          var tempEvalString = '';
          for (var logicEval in logicEvaluation) {
            if (index == 0) {
              tempEvalString = '${true} && ${logicEval['result']}';
            } else {
              tempEvalString = tempEvalString +
                  ' ${contionMapping[logicEval['join_condition']]} ${logicEval['result']}';
            }
            index += 1;
          }

          Expression exp = Expression(tempEvalString);
          // var finalResult =
          //     int.parse(exp.eval().toString()) == 0 ? false : true;

          var finalResult;
          if (tempEvalString.length > 0) {
            Expression exp = Expression(tempEvalString);
            finalResult = int.parse(exp.eval().toString()) == 0 ? false : true;
          } else {
            finalResult = true;
          }

          // var finalExp = exp.evaluate(EvaluationType.);
          return finalResult;
        } else {
          // return true since no logic is there for this question
          print('k');
          return true;
        }
      }

      // jumplogic
      if (type == 'jl') {
        print(
            'jump logic is called here j-id ${question['jumpLogic']['logics'][0]['logics']} ');
        if (question['jumpLogic']['logics'][0]['logics'].length > 0) {
          questionLogics = question['jumpLogic']['logics'][0]['logics'];
          print(
              "question logics for jump logic is ${questionLogics} comparaator");
          var logicEvaluation = [];
          for (var logic in questionLogics) {
            if (_allowedQuestionIds.contains(logic['question_id'])) {
              var join_condition = logic['join_condition'];
              print(
                  "logic to check is j-id ${logic['question_id']} ${logic['comparator']} ");
              var result = _checkingTheLogics(logic);
              ;
              logicEvaluation
                  .add({'join_condition': join_condition, 'result': result});
            }
          }
          print("jl logic logic evaluation is ${logicEvaluation}");
          var evalString;
          var index = 0;
          var tempEvalString = '';
          for (var logicEval in logicEvaluation) {
            if (index == 0) {
              tempEvalString = '${true} && ${logicEval['result']}';
            } else {
              tempEvalString = tempEvalString +
                  ' ${contionMapping[logicEval['join_condition']]} ${logicEval['result']}';
            }
            index += 1;
          }

          print("eval string for logic is ${tempEvalString}");
          var finalResult;
          if (tempEvalString.length > 0) {
            Expression exp = Expression(tempEvalString);
            finalResult = int.parse(exp.eval().toString()) == 0 ? false : true;
          } else {
            finalResult = false;
          }

          // var finalExp = exp.evaluate(EvaluationType.);
          return finalResult;
        } else {
          // return true since no logic is there for this question
          print('k');
          return true;
        }
      }
    }
  }

  _hasJumpLogic(question) {
    return question['jumpLogic']['logics'].length >= 1;
  }

  newJumpLogic(_currentId) {
    var nextIndex = _questionPos[_currentId] + 1;
    var questionToCheck = _allQuestionList[_questionPos[_currentId]];
    if (_hasJumpLogic(questionToCheck)) {
      print("called inside jump j-id");
      var questionIdToJump =
          questionToCheck['jumpLogic']['logics'][0]['jump_to_id'];
      var questionJumpLogics =
          questionToCheck['jumpLogic']['logics'][0]['logics'];
      var jump = _coreDisplayLogic(questionToCheck, 'jl');
      if (jump) {
        nextIndex = _questionPos[questionIdToJump];
      }
      print("jump is ${questionToCheck['id']} ${jump}");
    }

    return nextIndex;
  }

  newCheckDisplayLogic(questionId) {
    var canDisplay = false;
    var _currentId = questionId;
    var questionToCheck;
    while (!canDisplay) {
      var nextIndex = newJumpLogic(_currentId); // 1
      if (_allQuestionList.asMap()[nextIndex] != null) {
        questionToCheck = _allQuestionList[nextIndex]; // 67 question obj
        _currentId = questionToCheck['id'];
      } else {
        questionToCheck = null;
        _currentId = null;
      }
      canDisplay = _coreDisplayLogic(questionToCheck, 'dl');
    }
    if (questionToCheck != null) {
      return questionToCheck['id'];
    }
    return questionToCheck;
  }

  generateQuestionList(hadleNextSelect) {
    List<Map<dynamic, dynamic>> _questionsToConvert = [];
    var _questionArrayToConvert = [];
    var _currentQuestionIndex = _allQuestionList[0]['id'];

    do {
      _questionArrayToConvert.add(_currentQuestionIndex);
      _currentQuestionIndex = newCheckDisplayLogic(_currentQuestionIndex);
    } while (_currentQuestionIndex != null);

    for (var question in _questionArrayToConvert) {
      _questionsToConvert.add(_surveyToMap[question]);
    }

     _lastQuestion = _questionsToConvert.last;

    if (hadleNextSelect == true) {
      if (_pageNumber + 1 < _questionsToConvert.length) {
        var updatedQuestionNumber = _currentQuestionNumber + 1;
        _currentQuestionNumber = _currentQuestionNumber + 1;
        setState(() {
          _currentQuestionToRender = _questionsToConvert[_pageNumber + 1];
        });
        print(
            "checking current question in next -> 1109 ${_questionsToConvert[_pageNumber + 1]}");
      }
    }

    if (hadleNextSelect == false) {
      if (_pageNumber > 0) {
        var updatedQuestionNumber = _currentQuestionNumber - 1;
        _currentQuestionNumber = _currentQuestionNumber - 1;
        setState(() {
          _currentQuestionToRender = _questionsToConvert[_pageNumber - 1];
        });
        print(
            "checking current question in prevoius -> 1109 ${_questionsToConvert[_pageNumber - 1]}");
      }
    } 
    // print("1100p ${_answeredCount} ${_questionsToConvert.length} ${progressMadeSoFar}");
    setState(() {
      questionList = _convertQuestionListToWidget(_questionsToConvert);
      _progressMade = (_answeredCount / _questionsToConvert.length).toDouble();
    });
  }

  _handleNextQuestion() {
    generateQuestionList(true);
    if (_pageNumber + 1 < questionList.length) {
      var updatedPageNumber = _pageNumber + 1;
      // var updatedQuestionNumber = _currentQuestionNumber + 1;
      // print("119011 updated question number is ${updatedQuestionNumber}");

      setState(() {
        _pageNumber = updatedPageNumber;
        // _currentQuestionNumber = updatedQuestionNumber;
      });
      //       double _progressMade = 0.0;
      // int _answeredCount = 0;

      controller.animateToPage(updatedPageNumber,
          duration: Duration(milliseconds: 400), curve: Curves.easeIn);
    }
  }

  _handlePreviousQuestion() {
    generateQuestionList(false);
    if (_pageNumber != 0) {
      var updatedPageNumber = _pageNumber - 1;
      setState(() {
        _pageNumber = updatedPageNumber;
      });
      controller.animateToPage(updatedPageNumber,
          duration: Duration(milliseconds: 400), curve: Curves.easeIn);
    }
  }

  final PageController controller = PageController(initialPage: 0);
  @override
  Widget build(BuildContext context) {
    return Container(
      decoration: BoxDecoration(
        color: _surveyThemeClass.backgroundImage != 'noImage'
            ? _surveyThemeClass.backgroundImageColor
            : _surveyThemeClass.backgroundColor,
        image: _surveyThemeClass.backgroundImage != 'noImage'
            ? DecorationImage(
                colorFilter: ColorFilter.mode(
                    Colors.black.withOpacity(
                        _surveyThemeClass.backgroundImageColorOpacity),
                    BlendMode.dstATop),
                // colorFilter: ColorFilter.mode(Colors.black.withOpacity(0.1), BlendMode.dstATop),
                image: NetworkImage(_surveyThemeClass.backgroundImage),
                fit: BoxFit.cover)
            : null,
      ),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          LinearProgressIndicator(
            value: _progressMade,
            semanticsLabel: 'Linear progress indicator',
            backgroundColor: Colors.transparent,
            color: _surveyThemeClass.answerColor,
          ),
          Expanded(
            flex: 1,
            child: Container(
              padding: EdgeInsets.symmetric(horizontal: 24.0),
              child: PageView(
                physics: NeverScrollableScrollPhysics(),
                scrollDirection: Axis.vertical,
                controller: controller,
                children: <Widget>[
                  ...questionList,
                ],
              ),
            ),
          ),
          BottomNavigation(
            onClickNext: () {
              _handleNextQuestion();
            },
            onClickPrevious: () {
              _handlePreviousQuestion();
            },
          ),
        ],
      ),
    );
  }
}

Future<Map<dynamic, dynamic>> fetchAlbum() async {
  final response = await http.get(Uri.parse(
      'https://madbee.surveysparrow.com/api/internal/offline-app/v3/get-sdk-data'));
  print('inital load called');
  if (response.statusCode == 200) {
    // If the server did return a 200 OK response,
    // then parse the JSON.
    final parsedJson = jsonDecode(response.body);
    print('loaded parsed json ${parsedJson}');
    return parsedJson;
  } else {
    // If the server did not return a 200 OK response,
    // then throw an exception.
    throw Exception('Failed to load album');
  }
}



          // Container(
          //   width: double.infinity,
          //   color: Color.fromRGBO(247, 247, 247, 1),
          //   child: Container(
          //     margin: EdgeInsets.only(left: 10.0, right: 10.0, bottom: 20.0),
          //     child: Row(
          //       mainAxisAlignment: MainAxisAlignment.spaceBetween,
          //       crossAxisAlignment: CrossAxisAlignment.center,
          //       children: [
          //         SvgPicture.string(footerSVG),
          //         Container(
          //           child: Row(
          //             mainAxisAlignment: MainAxisAlignment.center,
          //             crossAxisAlignment: CrossAxisAlignment.center,
          //             children: [
          //               IconButton(
          //                   iconSize: 42,
          //                   onPressed: () {
          //                     generateQuestionList(false);
          //                     if (_pageNumber != 0) {
          //                       var updatedPageNumber = _pageNumber - 1;
          //                       setState(() {
          //                         _pageNumber = updatedPageNumber;
          //                       });
          //                       controller.animateToPage(updatedPageNumber,
          //                           duration: Duration(milliseconds: 400),
          //                           curve: Curves.easeIn);
          //                     }
          //                   },
          //                   icon: SvgPicture.string(upArrow)),
          //               IconButton(
          //                   iconSize: 42,
          //                   onPressed: () {
          //                     _handleNextQuestion();
          //                   },
          //                   icon: SvgPicture.string(downArrow)),
          //             ],
          //           ),
          //         ),
          //       ],
          //     ),
          //   ),
          // ),